"""Routines for low-level manipulation of binary data"""

from __future__ import annotations

from _typing import ReadableBuffer, WriteableBuffer

def bit_transpose(
    input: ReadableBuffer, output: WriteableBuffer, width: int = 8
) -> WriteableBuffer:
    """ "Transpose" a buffer by assembling each output byte with bits taken from each of ``width`` different input bytes.

    This can be useful to convert a sequence of pixel values into a single
    stream of bytes suitable for sending via a parallel conversion method.

    The number of bytes in the input buffer must be a multiple of the width,
    and the width can be any value from 2 to 8.  If the width is fewer than 8,
    then the remaining (less significant) bits of the output are set to zero.

    Let ``stride = len(input)//width``.  Then the first byte is made out of the
    most significant bits of ``[input[0], input[stride], input[2*stride], ...]``.
    The second byte is made out of the second bits, and so on until the 8th output
    byte which is made of the first bits of ``input[1], input[1+stride,
    input[2*stride], ...]``.

    The required output buffer size is ``len(input) * 8  // width``.

    Returns the output buffer."""
    ...



"""Board specific pin names

Common container for board base pin names. These will vary from board to
board so don't expect portability when using this module.

.. warning:: The board module varies by board. The APIs documented here may or may not be
             available on a specific board."""

from __future__ import annotations

import busio

def I2C() -> busio.I2C:
    """Returns the `busio.I2C` object for the board designated SDA and SCL pins. It is a singleton."""
    ...

def SPI() -> busio.SPI:
    """Returns the `busio.SPI` object for the board designated SCK, MOSI and MISO pins. It is a
    singleton."""
    ...

def UART() -> busio.UART:
    """Returns the `busio.UART` object for the board designated TX and RX pins. It is a singleton.

    The object created uses the default parameter values for `busio.UART`. If you need to set
    parameters that are not changeable after creation, such as ``receiver_buffer_size``,
    do not use `board.UART()`; instead create a `busio.UART` object explicitly with the
    desired parameters."""
    ...
